// $Id: drop_constituents.cc 2736 2007-10-20 16:59:04Z flaterco $

// Given a full set of constituents, make it look as it would have if
// the maxconstituents argument to harmgen had been specified.

// *** Overwrites the files. ***

#include "Dstr.hh"
#include <assert.h>

#define arraymax 255

int main (int argc, char **argv) {
  FILE *fp;
  unsigned maxc;
  if (argc < 3) {
    fprintf (stderr, "Usage: drop_constituents maxconstituents file...\n");
    return -1;
  }
  assert (sscanf (argv[1], "%u", &maxc) == 1);
  for (int looper=2; looper<argc; looper++) {
    if (!(fp = fopen (argv[looper], "r"))) {
      perror (argv[looper]);
    } else {
      Dstr header, buf;
      Dstr name[arraymax];
      double amp1[arraymax], amp2[arraymax], phase[arraymax];
      do {
        assert (!(buf.getline(fp).isNull()));
        header += buf;
        header += '\n';
      } while (buf[0] == '#');
      assert (!(buf.getline(fp).isNull()));
      header += buf;
      header += '\n';
      assert (!(buf.getline(fp).isNull()));
      header += buf;
      header += '\n';

      unsigned i;
      for (i=0; i<arraymax; i++)
        amp1[i] = phase[i] = amp2[i] = 0.0;
      unsigned numread = 0;

      while (!(buf.getline(fp).isNull())) {
        assert (numread < arraymax);
        buf /= name[numread];
        assert (sscanf (buf.aschar(), "%lf %lf", &(amp1[numread]), &(phase[numread])) == 2);
        numread++;
      }
      fclose (fp);

      for (i=0; i<maxc; i++) {
        int maxi = -1;
        double maxamp = 0.0;
        for (unsigned j=0; j<numread; j++) {
          if (amp1[j] > maxamp) {
            maxamp = amp1[j];
            maxi = j;
          }
        }
        if (maxi >= 0) {
          amp2[maxi] = amp1[maxi];
          amp1[maxi] = 0.0;
        } else
          break;
      }

      double maxdrop=0.0, totaldrop=0.0;
      for (i=0; i<numread; i++) {
	if (amp1[i] > maxdrop)
	  maxdrop = amp1[i];
	totaldrop += amp1[i];
      }

      Dstr fixheaders;
      header.getline (buf);
      fixheaders += buf;
      fixheaders += '\n';
      header.getline (buf);
      fixheaders += buf;
      fixheaders += '\n';
      header.getline (buf);
      fixheaders += buf;
      fixheaders += '\n';

      fixheaders += "# limit ";
      fixheaders += maxc;
      fixheaders += " out of ";
      fixheaders += numread;
      fixheaders += " constituents, ";
      if (totaldrop > 0.0) {
        char buf[80];
        sprintf (buf, "max dropped amp %.4f, total %.4f", maxdrop, totaldrop);
        fixheaders += buf;
      } else {
        fixheaders += "no loss";
      }
      fixheaders += '\n';
      fixheaders += header;

      if (!(fp = fopen (argv[looper], "w"))) {
        perror (argv[looper]);
      } else {
        fprintf (fp, "%s", fixheaders.aschar());
        for (i=0; i<numread; i++)
          if (amp2[i] > 0.0)
            fprintf (fp, "%-10s %10.4f %7.2f\n", name[i].aschar(), amp2[i], phase[i]);
          else
            fprintf (fp, "%-10s %10.4f %7.2f\n", name[i].aschar(), 0.0, 0.0);
        fclose (fp);
      }
    }
  }
  return 0;
}
